@@ -47,14 +47,14 @@ _ChunkSelectionMarkerMask = (
 class HgFileView(QFrame):
     "file diff, content, and annotation viewer"
 
-    linkActivated = pyqtSignal(QString)
-    fileDisplayed = pyqtSignal(QString, QString)
-    showMessage = pyqtSignal(QString)
+    linkActivated = pyqtSignal(str)
+    fileDisplayed = pyqtSignal(str, str)
+    showMessage = pyqtSignal(str)
     revisionSelected = pyqtSignal(int)
     shelveToolExited = pyqtSignal()
     chunkSelectionChanged = pyqtSignal()
 
-    grepRequested = pyqtSignal(unicode, dict)
+    grepRequested = pyqtSignal(str, dict)
     """Emitted (pattern, opts) when user request to search changelog"""
 
     def __init__(self, repoagent, parent):
@@ -201,7 +201,7 @@ class HgFileView(QFrame):
         for text in '12':
             a = self._parentToggleGroup.addAction(text)
             a.setCheckable(True)
-            a.setShortcut('Ctrl+%s' % text)
+            a.setShortcut('Ctrl+Shift+%s' % text)
 
         self.actionFind = self.searchbar.toggleViewAction()
         self.actionFind.setIcon(qtlib.geticon('edit-find'))
@@ -211,7 +211,7 @@ class HgFileView(QFrame):
                                     self._showSearchbar)
 
         self.actionShelf = QAction('Shelve', self)
-        self.actionShelf.setIcon(qtlib.geticon('shelve'))
+        self.actionShelf.setIcon(qtlib.geticon('hg-shelve'))
         self.actionShelf.setToolTip(_('Open shelve tool'))
         self.actionShelf.setVisible(False)
         self.actionShelf.triggered.connect(self._launchShelve)
@@ -404,7 +404,7 @@ class HgFileView(QFrame):
         if not self._fd.isNull():
             self._applyTextEncoding()
 
-    @pyqtSlot(unicode, int, int)
+    @pyqtSlot(str, int, int)
     def _setSource(self, path, rev, line):
         # BUG: not work for subrepo
         self.revisionSelected.emit(rev)
@@ -528,11 +528,11 @@ class HgFileView(QFrame):
             self.maxWidth += fm.width(longestline)
         self._updateScrollBar()
 
-    @pyqtSlot(unicode, bool, bool, bool)
+    @pyqtSlot(str, bool, bool, bool)
     def find(self, exp, icase=True, wrap=False, forward=True):
         self.sci.find(exp, icase, wrap, forward)
 
-    @pyqtSlot(unicode, bool)
+    @pyqtSlot(str, bool)
     def highlightText(self, match, icase=False):
         self._lastSearch = match, icase
         self.sci.highlightText(match, icase)
@@ -595,7 +595,7 @@ class HgFileView(QFrame):
         self.actionNextDiff.setEnabled(self._findNextChunk() >= 0)
         self.actionPrevDiff.setEnabled(self._findPrevChunk() >= 0)
 
-    @pyqtSlot(unicode, int, int)
+    @pyqtSlot(str, int, int)
     def _editSelected(self, path, rev, line):
         """Open editor to show the specified file"""
         path = hglib.fromunicode(path)
@@ -610,11 +610,11 @@ class HgFileView(QFrame):
         if dlg:
             dlg.exec_()
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def _visualDiffRevision(self, path, rev):
         self._visualDiff(path, change=rev)
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def _visualDiffToLocal(self, path, rev):
         self._visualDiff(path, rev=[str(rev)])
 
@@ -903,14 +903,14 @@ class _MessageViewControl(_AbstractViewControl):
 class _AnnotateViewControl(_AbstractViewControl):
     """Display annotation margin and colorize file content in HgFileView"""
 
-    showMessage = pyqtSignal(QString)
+    showMessage = pyqtSignal(str)
 
-    editSelectedRequested = pyqtSignal(unicode, int, int)
-    grepRequested = pyqtSignal(unicode, dict)
+    editSelectedRequested = pyqtSignal(str, int, int)
+    grepRequested = pyqtSignal(str, dict)
     searchSelectedTextRequested = pyqtSignal()
-    setSourceRequested = pyqtSignal(unicode, int, int)
-    visualDiffRevisionRequested = pyqtSignal(unicode, int)
-    visualDiffToLocalRequested = pyqtSignal(unicode, int)
+    setSourceRequested = pyqtSignal(str, int, int)
+    visualDiffRevisionRequested = pyqtSignal(str, int)
+    visualDiffToLocalRequested = pyqtSignal(str, int)
 
     def __init__(self, repoagent, sci, fd, parent=None):
         super(_AnnotateViewControl, self).__init__(parent)
@@ -959,7 +959,7 @@ class _AnnotateViewControl(_AbstractViewControl):
         wb = "Annotate/"
         for a in self._annoptactions:
             a.setChecked(s.value(wb + a.data().toString()).toBool())
-        if not util.any(a.isChecked() for a in self._annoptactions):
+        if not any(a.isChecked() for a in self._annoptactions):
             self._annoptactions[-1].setChecked(True)  # 'rev' by default
 
     def _saveAnnotateSettings(self):
@@ -981,7 +981,7 @@ class _AnnotateViewControl(_AbstractViewControl):
     @pyqtSlot()
     def _updateAnnotateOption(self):
         # make sure at least one option is checked
-        if not util.any(a.isChecked() for a in self._annoptactions):
+        if not any(a.isChecked() for a in self._annoptactions):
             self.sender().setChecked(True)
 
         self._updateView()
@@ -1249,10 +1249,12 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
         self._sci.markerDefine(p, _ExcludedChunkStartMarker)
 
         self._sci.markerDefine(qsci.Background, _ExcludedLineMarker)
-        self._sci.setMarkerBackgroundColor(QColor('lightgrey'),
-                                           _ExcludedLineMarker)
-        self._sci.setMarkerForegroundColor(QColor('darkgrey'),
-                                           _ExcludedLineMarker)
+        if qtlib.isDarkTheme(self._sci.palette()):
+            bg, fg = QColor(44, 44, 44), QColor(86, 86, 86)
+        else:
+            bg, fg = QColor('lightgrey'), QColor('darkgrey')
+        self._sci.setMarkerBackgroundColor(bg, _ExcludedLineMarker)
+        self._sci.setMarkerForegroundColor(fg, _ExcludedLineMarker)
         self._sci.setMarginType(_ChunkSelectionMargin, qsci.SymbolMargin)
         self._sci.setMarginMarkerMask(_ChunkSelectionMargin,
                                       _ChunkSelectionMarkerMask)
