@@ -5,6 +5,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+import cStringIO
+import glob
 import os
 import re
 import sys
@@ -14,25 +16,22 @@ import time
 from mercurial import ui, util, extensions
 from mercurial import encoding, templatefilters, filemerge, error, pathutil
 from mercurial import dispatch as dispatchmod
+from mercurial import merge as mergemod
 from mercurial import revset as revsetmod
 from mercurial.node import nullrev
 from hgext import mq as mqmod
 
-try:
-    from mercurial import patch
-    parsepatch = patch.parsepatch
-    patchheader = patch.header
-except AttributeError:
-    # hg<3.4 (20aac24e2114, dc655360bccb)
-    from hgext import record
-    parsepatch = record.parsepatch
-    patchheader = record.header
-
 _encoding = encoding.encoding
 _fallbackencoding = encoding.fallbackencoding
 
 # extensions which can cause problem with TortoiseHg
-_extensions_blacklist = ('color', 'pager', 'progress', 'zeroconf')
+_extensions_blacklist = (
+    'blackbox',  # mucks uimod.ui (hg 851c41a21869, issue #4489)
+    'color',
+    'pager',
+    'progress',
+    'zeroconf',
+)
 
 from tortoisehg.util import paths
 from tortoisehg.util.hgversion import hgversion
@@ -114,6 +113,9 @@ def fromutf(s):
         return str(fromunicode(s.decode('utf-8', 'replace'), 'replace'))
 
 
+def activebookmark(repo):
+    return repo._activebookmark
+
 def namedbranches(repo):
     branchmap = repo.branchmap()
     dead = repo.deadbranches
@@ -143,10 +145,10 @@ def _getfirstrevisionlabel(repo, ctx):
     # see context.changectx for look-up order of labels
 
     bookmarks = ctx.bookmarks()
-    if ctx in repo.parents():
+    if ctx in repo[None].parents():
         # keep bookmark unchanged when updating to current rev
-        if repo._bookmarkcurrent in bookmarks:
-            return repo._bookmarkcurrent
+        if activebookmark(repo) in bookmarks:
+            return activebookmark(repo)
     else:
         # more common switching bookmark, rather than deselecting it
         if bookmarks:
@@ -201,6 +203,8 @@ def getqqueues(repo):
         qqueues = []
     return qqueues
 
+readmergestate = mergemod.mergestate.read
+
 def readundodesc(repo):
     """Read short description and changelog size of last transaction"""
     if os.path.exists(repo.sjoin('undo')):
@@ -266,11 +270,30 @@ def _loadextensionwithblacklist(orig, ui, name, path):
 
     return orig(ui, name, path)
 
-def wrapextensionsloader():
+def _wrapextensionsloader():
     """Wrap extensions.load(ui, name) for blacklist to take effect"""
     extensions.wrapfunction(extensions, 'load',
                             _loadextensionwithblacklist)
 
+def loadextensions(ui):
+    """Load and setup extensions for GUI process"""
+    _wrapextensionsloader()  # enable blacklist of extensions
+    loadedbefore = set(name for name, module in extensions.extensions())
+    extensions.loadall(ui)
+
+    # we still generate diffs and commit descriptions in GUI process, which
+    # may require template functions loaded from extensions; so run loaders
+    # of template* tables just like dispatch._dispatch(). (issue #4515)
+    for name, module in extensions.extensions():
+        if name in loadedbefore:
+            continue
+        for objname, loadermod, loadername in dispatchmod.extraloaders:
+            if not objname.startswith('template'):
+                continue
+            extraobj = getattr(module, objname, None)
+            if extraobj is not None:
+                getattr(loadermod, loadername)(ui, name, extraobj)
+
 # TODO: provide singular canonpath() wrapper instead?
 def canonpaths(list):
     'Get canonical paths (relative to root) for list of files'
@@ -560,6 +583,26 @@ def shortreponame(ui):
         return
     return name
 
+def extractchoices(prompttext):
+    """Extract prompt message and list of choice (char, label) pairs
+
+    This is slightly different from ui.extractchoices() in that
+    a. prompttext may be a unicode
+    b. choice label includes &-accessor
+
+    >>> extractchoices("awake? $$ &Yes $$ &No")
+    ('awake? ', [('y', '&Yes'), ('n', '&No')])
+    >>> extractchoices("line\\nbreak? $$ &Yes $$ &No")
+    ('line\\nbreak? ', [('y', '&Yes'), ('n', '&No')])
+    >>> extractchoices("want lots of $$money$$?$$Ye&s$$N&o")
+    ('want lots of $$money$$?', [('s', 'Ye&s'), ('o', 'N&o')])
+    """
+    m = re.match(r'(?s)(.+?)\$\$([^\$]*&[^ \$].*)', prompttext)
+    msg = m.group(1)
+    choices = [p.strip(' ') for p in m.group(2).split('$$')]
+    resps = [p[p.index('&') + 1].lower() for p in choices]
+    return msg, zip(resps, choices)
+
 def displaytime(date):
     return util.datestr(date, '%Y-%m-%d %H:%M:%S %1%2')
 
@@ -840,7 +883,7 @@ def buildcmdargs(name, *args, **opts):
             fullargs.append(_stringify(v))
 
     args = [_stringify(v) for v in args if v is not None]
-    if util.any(e.startswith('-') for e in args):
+    if any(e.startswith('-') for e in args):
         fullargs.append('--')
     fullargs.extend(args)
 
@@ -874,3 +917,65 @@ def prettifycmdline(cmdline):
     'clone svn+http://:***@example.org:8080/trunk/'
     """
     return ' '.join(_reprcmdarg(e) for e in cmdline)
+
+def parsecmdline(cmdline, cwd):
+    r"""Split command line string to imitate a unix shell
+
+    >>> origfuncs = glob.glob, os.path.expanduser, os.path.expandvars
+    >>> glob.glob = lambda p: [p.replace('*', e) for e in ['foo', 'bar', 'baz']]
+    >>> os.path.expanduser = lambda p: re.sub(r'^~', '/home/foo', p)
+    >>> os.path.expandvars = lambda p: p.replace('$var', 'bar')
+
+    emulates glob/variable expansion rule for simple cases:
+
+    >>> parsecmdline('foo * "qux quux" "*"  "*"', '.')
+    [u'foo', u'foo', u'bar', u'baz', u'qux quux', u'*', u'*']
+    >>> parsecmdline('foo /*', '.')
+    [u'foo', u'/foo', u'/bar', u'/baz']
+    >>> parsecmdline('''foo ~/bar '~/bar' "~/bar"''', '.')
+    [u'foo', u'/home/foo/bar', u'~/bar', u'~/bar']
+    >>> parsecmdline('''foo $var '$var' "$var"''', '.')
+    [u'foo', u'bar', u'$var', u'bar']
+
+    but the following cases are unsupported:
+
+    >>> parsecmdline('"foo"*"bar"', '.')  # '*' should be expanded
+    [u'foo*bar']
+    >>> parsecmdline(r'\*', '.')  # '*' should be a literal
+    [u'foo', u'bar', u'baz']
+
+    >>> glob.glob, os.path.expanduser, os.path.expandvars = origfuncs
+    """
+    _ = _gettext  # TODO: use unicode version globally
+    # shlex can't process unicode on Python < 2.7.3
+    cmdline = cmdline.encode('utf-8')
+    src = cStringIO.StringIO(cmdline)
+    lex = shlex.shlex(src, posix=True)
+    lex.whitespace_split = True
+    lex.commenters = ''
+    args = []
+    while True:
+        # peek first char of next token to guess its type. this isn't perfect
+        # but can catch common cases.
+        q = cmdline[src.tell():].lstrip(lex.whitespace)[:1]
+        try:
+            e = lex.get_token()
+        except ValueError as err:
+            raise ValueError(_('command parse error: %s') % err)
+        if e == lex.eof:
+            return args
+        e = e.decode('utf-8')
+        if q not in lex.quotes or q in lex.escapedquotes:
+            e = os.path.expandvars(e)  # $var or "$var"
+        if q not in lex.quotes:
+            e = os.path.expanduser(e)  # ~user
+        if q not in lex.quotes and any(c in e for c in '*?[]'):
+            expanded = glob.glob(os.path.join(cwd, e))
+            if not expanded:
+                raise ValueError(_('no matches found: %s') % e)
+            if os.path.isabs(e):
+                args.extend(expanded)
+            else:
+                args.extend(p[len(cwd) + 1:] for p in expanded)
+        else:
+            args.append(e)
